/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "user_data.hh"
#include <fstream>
#include <stdexcept>
#include <cstdlib>

using namespace sends;
using namespace std;

static const int linebuf_size=128;

inline void
purge_string(string& s) {
    long N = s.size();
    for (long i=0; i<N; ++i) s[i] = '\0';
}

//======================================  User data Constructor.
user_data::user_data(const string& name, const string& pwd, long access)
    : mUserName(name), mKeyWord(pwd), mAccess(access)
{
}

//======================================  User data destructor.
user_data::~user_data(void) {
    erase();
}

//======================================  Erase sensitive user data.
void
user_data::erase(void) {
    purge_string(mUserName);
    purge_string(mKeyWord);
    mAccess = 0;
}

//======================================  User list Constructor.
user_list::user_list(void) {
}

//======================================  User list destructor
user_list::~user_list(void) {
}

//======================================  Local slurp function
ostream& 
user_list::dumpUsers(ostream& out) const {
    for (const_user_iterator i=mUserMap.begin(); i!=mUserMap.end(); ++i) {
	out << i->second.user() << ",  " << i->second.access() << endl;
    }
    return out;
}

//======================================  Local slurp function
static const char*
slurp(const char* p0, string& out) {
    const char* p  = p0;
    while (*p && *p != ':') p++;
    out = string(p0, p-p0);
    return p;
}

//======================================  Read data from a user file.
void 
user_list::readFile(const std::string& file) {
    ifstream in(file.c_str());
    if (!in.is_open()) {
	throw runtime_error(string("Unable to open: ")+file);
    }

    char buf[linebuf_size];
    string user, key;
    long access;
    while (in.good()) {
	in.getline(buf, sizeof(buf)-1);
	int len = in.gcount();
	if (len <= 0) continue;
	buf[len]=0;
	if (*buf == '#') continue;

	const char* p0 = buf;
	p0 = slurp(p0, user);
	p0++;
	
	p0 = slurp(p0, key);
	p0++;
	
	access = strtol(p0, 0, 0);
	user_map::value_type node(user, user_data(user, key, access));
	mUserMap.insert(node);
    }

    for (long i=0; i<linebuf_size; ++i) buf[i] = '\0';
    purge_string(user);
    purge_string(key);
}

//======================================  Pick a specified user
const user_data& 
user_list::operator[](const std::string& name) const {
    const_user_iterator p = mUserMap.find(name);
    return p->second;
}
